@@ -1,12 +1,13 @@
-/// A library for parsing and manipulating R DESCRIPTION files.
-///
-/// See https://r-pkgs.org/description.html and https://cran.r-project.org/doc/manuals/R-exts.html
-/// for more information
-///
-/// See the ``lossless`` module for a lossless parser that is
-/// forgiving in the face of errors and preserves formatting while editing
-/// at the expense of a more complex API.
-use deb822_lossless::{FromDeb822, FromDeb822Paragraph, ToDeb822, ToDeb822Paragraph};
+//! A library for parsing and manipulating R DESCRIPTION files.
+//!
+//! See https://r-pkgs.org/description.html and https://cran.r-project.org/doc/manuals/R-exts.html
+//! for more information
+//!
+//! See the ``lossless`` module for a lossless parser that is
+//! forgiving in the face of errors and preserves formatting while editing
+//! at the expense of a more complex API.
+use deb822_fast::{FromDeb822Paragraph, ToDeb822Paragraph};
+use deb822_derive::{FromDeb822, ToDeb822};
 
 use crate::RCode;
 use std::iter::Peekable;
@@ -16,8 +17,12 @@ use crate::relations::{lex, SyntaxKind, VersionConstraint};
 use crate::version::Version;
 
 #[derive(Debug, Clone, PartialEq, Eq)]
+/// A URL entry in the URL field.
 pub struct UrlEntry {
+    /// URL
     pub url: url::Url,
+
+    /// Optional label for the URL.
     pub label: Option<String>,
 }
 
@@ -25,7 +30,7 @@ impl std::fmt::Display for UrlEntry {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "{}", self.url.as_str())?;
         if let Some(label) = &self.label {
-            write!(f, " ({})", label)?;
+            write!(f, " ({label})")?;
         }
         Ok(())
     }
@@ -71,29 +76,35 @@ fn deserialize_url_list(s: &str) -> Result<Vec<UrlEntry>, String> {
 }
 
 #[derive(FromDeb822, ToDeb822, Debug, PartialEq, Eq)]
+/// A DESCRIPTION file.
 pub struct RDescription {
+    /// The name of the package.
     #[deb822(field = "Package")]
     pub name: String,
 
+    /// A short description of the package.
     #[deb822(field = "Description")]
     pub description: String,
 
     #[deb822(field = "Title")]
+    /// The title of the package.
     pub title: String,
 
     #[deb822(field = "Maintainer")]
+    /// The maintainer of the package.
     pub maintainer: Option<String>,
 
     #[deb822(field = "Author")]
     /// Who wrote the the package
     pub author: Option<String>,
 
-    // 'Authors@R' is a special field that can contain R code
-    // that is evaluated to get the authors and maintainers.
+    /// 'Authors@R' is a special field that can contain R code
+    /// that is evaluated to get the authors and maintainers.
     #[deb822(field = "Authors@R")]
     pub authors: Option<RCode>,
 
     #[deb822(field = "Version")]
+    /// The version of the package.
     pub version: Version,
 
     /// If the DESCRIPTION file is not written in pure ASCII, the encoding
@@ -102,37 +113,48 @@ pub struct RDescription {
     pub encoding: Option<String>,
 
     #[deb822(field = "License")]
+    /// The license of the package.
     pub license: String,
 
     #[deb822(field = "URL", serialize_with = serialize_url_list, deserialize_with = deserialize_url_list)]
     // TODO: parse this as a list of URLs, separated by commas
+    /// URLs related to the package.
     pub url: Option<Vec<UrlEntry>>,
 
     #[deb822(field = "BugReports")]
+    /// The URL or email address where bug reports should be sent.
     pub bug_reports: Option<String>,
 
     #[deb822(field = "Imports")]
+    /// The packages that this package depends on.
     pub imports: Option<Relations>,
 
     #[deb822(field = "Suggests")]
+    /// The packages that this package suggests.
     pub suggests: Option<Relations>,
 
     #[deb822(field = "Depends")]
+    /// The packages that this package depends on.
     pub depends: Option<Relations>,
 
     #[deb822(field = "LinkingTo")]
+    /// The packages that this package links to.
     pub linking_to: Option<Relations>,
 
     #[deb822(field = "LazyData")]
+    /// Whether the package has lazy data.
     pub lazy_data: Option<String>,
 
     #[deb822(field = "Collate")]
+    /// The order in which R scripts are loaded.
     pub collate: Option<String>,
 
     #[deb822(field = "VignetteBuilder")]
+    /// The package used to build vignettes.
     pub vignette_builder: Option<String>,
 
     #[deb822(field = "SystemRequirements")]
+    /// The system requirements for the package.
     pub system_requirements: Option<String>,
 
     #[deb822(field = "Date")]
@@ -215,7 +237,7 @@ impl std::fmt::Display for Relation {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "{}", self.name)?;
         if let Some((constraint, version)) = &self.version {
-            write!(f, " ({} {})", constraint, version)?;
+            write!(f, " ({constraint} {version})")?;
         }
         Ok(())
     }
@@ -313,7 +335,7 @@ impl std::fmt::Display for Relations {
             if i > 0 {
                 f.write_str(", ")?;
             }
-            write!(f, "{}", relation)?;
+            write!(f, "{relation}")?;
         }
         Ok(())
     }
@@ -378,7 +400,7 @@ impl std::str::FromStr for Relation {
         eat_whitespace(&mut tokens);
 
         if let Some((kind, _)) = tokens.next() {
-            return Err(format!("Unexpected token: {:?}", kind));
+            return Err(format!("Unexpected token: {kind:?}"));
         }
 
         Ok(Relation { name, version })
@@ -439,7 +461,7 @@ impl std::str::FromStr for RDescription {
 
 impl std::fmt::Display for RDescription {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        let para: deb822_lossless::lossy::Paragraph = self.to_paragraph();
+        let para: deb822_lossless::Paragraph = self.to_paragraph();
         f.write_str(&para.to_string())?;
         Ok(())
     }
